@@ -18,9 +18,10 @@ as device topics by MQTT.
 """
 
 AP_EPILOG="""
-It is strongly recommended to run rtl_433 with "-C si" and "-M newmodel".
+It is strongly recommended to run rtl_433 with "-C si".
 This script requires rtl_433 to publish both event messages and device
-messages.
+messages. If you've changed the device topic in rtl_433, use the same device
+topic with the "-T" parameter.
 
 MQTT Username and Password can be set via the cmdline or passed in the
 environment: MQTT_USERNAME and MQTT_PASSWORD.
@@ -60,6 +61,9 @@ recent value even if not changed set -f to append "force_update = true" to
 all configs. This is useful if you're graphing the sensor data or want to
 alert on missing data.
 
+If you have changed the topic structure from the default topics in the rtl433
+configuration use the -T parameter to set the same topic structure here.
+
 Suggestions:
 
 Running this script will cause a number of Home Assistant entities (sensors
@@ -68,7 +72,7 @@ the topic is republished with an empty config string.  To avoid having to
 do a lot of clean up When running this initially or debugging, set this
 script to publish to a topic other than the one Home Assistant users (homeassistant).
 
-MQTT Explorer (http://http://mqtt-explorer.com/) is a very nice GUI for
+MQTT Explorer (http://mqtt-explorer.com/) is a very nice GUI for
 working with MQTT. It is free, cross platform, and OSS. The structured
 hierarchical view makes it easier to understand what rtl_433 is publishing
 and how this script works with Home Assistant.
@@ -96,17 +100,15 @@ import logging
 import time
 import json
 import paho.mqtt.client as mqtt
+import re
 
 
 discovery_timeouts = {}
 
-# Fields used for creating topic names
-NAMING_KEYS = [ "type", "model", "subtype", "channel", "id" ]
-
 # Fields that get ignored when publishing to Home Assistant
 # (reduces noise to help spot missing field mappings)
-SKIP_KEYS = NAMING_KEYS + [ "mic", "mod", "freq", "sequence_num",
-                            "message_type", "exception", "raw_msg" ]
+SKIP_KEYS = [ "type", "model", "subtype", "channel", "id", "mic", "mod",
+                "freq", "sequence_num", "message_type", "exception", "raw_msg" ]
 
 
 # Global mapping of rtl_433 field names to Home Assistant metadata.
@@ -188,6 +190,19 @@ mappings = {
         }
     },
 
+    "battery_mV": {
+        "device_type": "sensor",
+        "object_suffix": "mV",
+        "config": {
+            "device_class": "voltage",
+            "name": "Battery mV",
+            "unit_of_measurement": "mV",
+            "value_template": "{{ float(value) }}",
+            "state_class": "measurement",
+            "entity_category": "diagnostic"
+        }
+    },
+
     "humidity": {
         "device_type": "sensor",
         "object_suffix": "H",
@@ -199,12 +214,34 @@ mappings = {
             "state_class": "measurement"
         }
     },
+    "humidity_1": {
+        "device_type": "sensor",
+        "object_suffix": "H1",
+        "config": {
+            "device_class": "humidity",
+            "name": "Humidity 1",
+            "unit_of_measurement": "%",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
+    "humidity_2": {
+        "device_type": "sensor",
+        "object_suffix": "H2",
+        "config": {
+            "device_class": "humidity",
+            "name": "Humidity 2",
+            "unit_of_measurement": "%",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
 
     "moisture": {
         "device_type": "sensor",
-        "object_suffix": "H",
+        "object_suffix": "M",
         "config": {
-            "device_class": "humidity",
+            "device_class": "moisture",
             "name": "Moisture",
             "unit_of_measurement": "%",
             "value_template": "{{ value|float }}",
@@ -212,6 +249,18 @@ mappings = {
         }
     },
 
+    "detect_wet": {
+        "device_type": "binary_sensor",
+        "object_suffix": "moisture",
+        "config": {
+            "name": "Water Sensor",
+            "device_class": "moisture",
+            "force_update": "true",
+            "payload_on": "1",
+            "payload_off": "0"
+        }
+    },
+
     "pressure_hPa": {
         "device_type": "sensor",
         "object_suffix": "P",
@@ -240,6 +289,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ value|float }}",
@@ -251,6 +301,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ value|float }}",
@@ -262,6 +313,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind Speed",
             "unit_of_measurement": "mi/h",
             "value_template": "{{ value|float }}",
@@ -273,6 +325,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind Average",
             "unit_of_measurement": "km/h",
             "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
@@ -284,6 +337,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ float(value|float) * 3.6 }}",
@@ -295,6 +349,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Gust Speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ value|float }}",
@@ -306,6 +361,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind max speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ value|float }}",
@@ -317,6 +373,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Wind max",
             "unit_of_measurement": "km/h",
             "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
@@ -328,6 +385,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
+            "device_class": "wind_speed",
             "name": "Gust Speed",
             "unit_of_measurement": "km/h",
             "value_template": "{{ float(value|float) * 3.6 }}",
@@ -350,6 +408,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "RT",
         "config": {
+            "device_class": "precipitation",
             "name": "Rain Total",
             "unit_of_measurement": "mm",
             "value_template": "{{ value|float|round(2) }}",
@@ -357,10 +416,11 @@ mappings = {
         }
     },
 
-    "rain_mm_h": {
+    "rain_rate_mm_h": {
         "device_type": "sensor",
         "object_suffix": "RR",
         "config": {
+            "device_class": "precipitation_intensity",
             "name": "Rain Rate",
             "unit_of_measurement": "mm/h",
             "value_template": "{{ value|float }}",
@@ -372,6 +432,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "RT",
         "config": {
+            "device_class": "precipitation",
             "name": "Rain Total",
             "unit_of_measurement": "mm",
             "value_template": "{{ (float(value|float) * 25.4) | round(2) }}",
@@ -383,6 +444,7 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "RR",
         "config": {
+            "device_class": "precipitation_intensity",
             "name": "Rain Rate",
             "unit_of_measurement": "mm/h",
             "value_template": "{{ (float(value|float) * 25.4) | round(2) }}",
@@ -472,13 +534,50 @@ mappings = {
             "state_class": "measurement"
         }
     },
+  
+    "energy_kWh": {
+        "device_type": "sensor",
+        "object_suffix": "kwh",
+        "config": {
+            "device_class": "power",
+            "name": "Energy",
+            "unit_of_measurement": "kWh",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
+  
+    "current_A": {
+        "device_type": "sensor",
+        "object_suffix": "A",
+        "config": {
+            "device_class": "current",
+            "name": "Current",
+            "unit_of_measurement": "A",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
+  
+    "voltage_V": {
+        "device_type": "sensor",
+        "object_suffix": "V",
+        "config": {
+            "device_class": "power",
+            "name": "Voltage",
+            "unit_of_measurement": "V",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
 
     "light_lux": {
         "device_type": "sensor",
         "object_suffix": "lux",
         "config": {
+            "device_class": "illuminance",
             "name": "Outside Luminance",
-            "unit_of_measurement": "lux",
+            "unit_of_measurement": "lx",
             "value_template": "{{ value|int }}",
             "state_class": "measurement"
         }
@@ -487,8 +586,9 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "lux",
         "config": {
+            "device_class": "illuminance",
             "name": "Outside Luminance",
-            "unit_of_measurement": "lux",
+            "unit_of_measurement": "lx",
             "value_template": "{{ value|int }}",
             "state_class": "measurement"
         }
@@ -556,6 +656,16 @@ mappings = {
             "state_class": "total_increasing",
         }
     },
+  
+    "consumption": {
+        "device_type": "sensor",
+        "object_suffix": "consumption",
+        "config": {
+            "name": "SCMplus Consumption Value",
+            "value_template": "{{ value|int }}",
+            "state_class": "total_increasing",
+        }
+    },
 
     "channel": {
         "device_type": "device_automation",
@@ -608,6 +718,8 @@ secret_knock_mappings = [
 
 ]
 
+TOPIC_PARSE_RE = re.compile(r'\[(?P<slash>/?)(?P<token>[^\]:]+):?(?P<default>[^\]:]*)\]')
+
 def mqtt_connect(client, userdata, flags, rc):
     """Callback for MQTT connects."""
 
@@ -648,28 +760,41 @@ def sanitize(text):
             .replace(".", "_")
             .replace("&", ""))
 
-def rtl_433_device_topic(data):
-    """Return rtl_433 device topic to subscribe to for a data element"""
+def rtl_433_device_info(data, topic_prefix):
+    """Return rtl_433 device topic to subscribe to for a data element, based on the
+    rtl_433 device topic argument, as well as the device identifier"""
 
     path_elements = []
-
-    for key in NAMING_KEYS:
+    id_elements = []
+    last_match_end = 0
+    # The default for args.device_topic_suffix is the same topic structure
+    # as set by default in rtl433 config
+    for match in re.finditer(TOPIC_PARSE_RE, args.device_topic_suffix):
+        path_elements.append(args.device_topic_suffix[last_match_end:match.start()])
+        key = match.group(2)
         if key in data:
+            # If we have this key, prepend a slash if needed
+            if match.group(1):
+                path_elements.append('/')
             element = sanitize(str(data[key]))
             path_elements.append(element)
+            id_elements.append(element)
+        elif match.group(3):
+            path_elements.append(match.group(3))
+        last_match_end = match.end()
 
-    return '/'.join(path_elements)
+    path = ''.join(list(filter(lambda item: item, path_elements)))
+    id = '-'.join(id_elements)
+    return (f"{topic_prefix}/{path}", id)
 
 
-def publish_config(mqttc, topic, model, instance, mapping, value=None):
+def publish_config(mqttc, topic, model, object_id, mapping, key=None):
     """Publish Home Assistant auto discovery data."""
     global discovery_timeouts
 
-    instance_no_slash = instance.replace("/", "-")
     device_type = mapping["device_type"]
     object_suffix = mapping["object_suffix"]
-    object_id = instance_no_slash
-    object_name = "-".join([object_id,object_suffix])
+    object_name = "-".join([object_id, object_suffix])
 
     path = "/".join([args.discovery_prefix, device_type, object_id, object_name, "config"])
 
@@ -691,9 +816,10 @@ def publish_config(mqttc, topic, model, instance, mapping, value=None):
         config["topic"] = topic
         config["platform"] = 'mqtt'
     else:
+        readable_name = mapping["config"]["name"] if "name" in mapping["config"] else key
         config["state_topic"] = topic
         config["unique_id"] = object_name
-        config["name"] = object_name
+        config["name"] = readable_name
     config["device"] = { "identifiers": [object_id], "name": object_id, "model": model, "manufacturer": "rtl_433" }
 
     if args.force_update:
@@ -708,7 +834,7 @@ def publish_config(mqttc, topic, model, instance, mapping, value=None):
 
     return True
 
-def bridge_event_to_hass(mqttc, topicprefix, data):
+def bridge_event_to_hass(mqttc, topic_prefix, data):
     """Translate some rtl_433 sensor data to Home Assistant auto discovery."""
 
     if "model" not in data:
@@ -721,13 +847,13 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
     skipped_keys = []
     published_keys = []
 
-    instance = rtl_433_device_topic(data)
-    if not instance:
+    base_topic, device_id = rtl_433_device_info(data, topic_prefix)
+    if not device_id:
         # no unique device identifier
-        logging.warning("No suitable identifier found for model: ", model)
+        logging.warning("No suitable identifier found for model: %s", model)
         return
 
-    if args.ids and id in data and data.get("id") not in args.ids:
+    if args.ids and "id" in data and data.get("id") not in args.ids:
         # not in the safe list
         logging.debug("Device (%s) is not in the desired list of device ids: [%s]" % (data["id"], ids))
         return
@@ -736,8 +862,8 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
     for key in data.keys():
         if key in mappings:
             # topic = "/".join([topicprefix,"devices",model,instance,key])
-            topic = "/".join([topicprefix,"devices",instance,key])
-            if publish_config(mqttc, topic, model, instance, mappings[key]):
+            topic = "/".join([base_topic, key])
+            if publish_config(mqttc, topic, model, device_id, mappings[key], key):
                 published_keys.append(key)
         else:
             if key not in SKIP_KEYS:
@@ -745,15 +871,15 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
 
     if "secret_knock" in data.keys():
         for m in secret_knock_mappings:
-            topic = "/".join([topicprefix,"devices",instance,"secret_knock"])
-            if publish_config(mqttc, topic, model, instance, m):
+            topic = "/".join([base_topic, "secret_knock"])
+            if publish_config(mqttc, topic, model, device_id, m, "secret_knock"):
                 published_keys.append("secret_knock")
 
     if published_keys:
-        logging.info("Published %s: %s" % (instance, ", ".join(published_keys)))
+        logging.info("Published %s: %s" % (device_id, ", ".join(published_keys)))
 
         if skipped_keys:
-            logging.info("Skipped %s: %s" % (instance, ", ".join(skipped_keys)))
+            logging.info("Skipped %s: %s" % (device_id, ", ".join(skipped_keys)))
 
 
 def rtl_433_bridge():
@@ -792,6 +918,7 @@ def run():
 
 
 if __name__ == "__main__":
+    logging.basicConfig(format='[%(asctime)s] %(levelname)s:%(name)s:%(message)s',datefmt='%Y-%m-%dT%H:%M:%S%z')
     logging.getLogger().setLevel(logging.INFO)
 
     parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
@@ -818,6 +945,11 @@ if __name__ == "__main__":
                         dest="discovery_prefix",
                         default="homeassistant",
                         help="Home Assistant MQTT topic prefix (default: %(default)s)")
+    # This defaults to the rtl433 config default, so we assemble the same topic structure
+    parser.add_argument("-T", "--device-topic_suffix", type=str,
+                        dest="device_topic_suffix",
+                        default="devices[/type][/model][/subtype][/channel][/id]",
+                        help="rtl_433 device topic suffix (default: %(default)s)")
     parser.add_argument("-i", "--interval", type=int,
                         dest="discovery_interval",
                         default=600,
