@@ -58,20 +58,22 @@ typedef struct data_array {
     void        *values;
 } data_array_t;
 
+// Note: Do not unwrap a packed array to data_value_t,
+// on 32-bit the union has different size/alignment than a pointer.
 typedef union data_value {
-    int         v_int;
-    double      v_dbl;
-    void        *v_ptr;
+    int         v_int;  /**< A data value of type int, 4 bytes size/alignment */
+    double      v_dbl;  /**< A data value of type double, 8 bytes size/alignment */
+    void        *v_ptr; /**< A data value pointer, 4/8 bytes size/alignment */
 } data_value_t;
 
 typedef struct data {
+    struct data *next; /**< chaining to the next element in the linked list; NULL indicates end-of-list */
     char        *key;
     char        *pretty_key; /**< the name used for displaying data to user in with a nicer name */
-    data_type_t type;
     char        *format; /**< if not null, contains special formatting string */
     data_value_t value;
+    data_type_t type;
     unsigned    retain; /**< incremented on data_retain, data_free only frees if this is zero */
-    struct data *next; /**< chaining to the next element in the linked list; NULL indicates end-of-list */
 } data_t;
 
 /** Constructs a structured data object.
@@ -128,7 +130,7 @@ R_API data_t *data_prepend(data_t *first, const char *key, const char *pretty_ke
     @return The constructed data array object, typically placed inside a data_t or NULL
             if there was a memory allocation error.
 */
-R_API data_array_t *data_array(int num_values, data_type_t type, void *ptr);
+R_API data_array_t *data_array(int num_values, data_type_t type, void const *ptr);
 
 /** Releases a data array. */
 R_API void data_array_free(data_array_t *array);
@@ -148,8 +150,9 @@ typedef struct data_output {
     void (R_API_CALLCONV *print_double)(struct data_output *output, double data, char const *format);
     void (R_API_CALLCONV *print_int)(struct data_output *output, int data, char const *format);
     void (R_API_CALLCONV *output_start)(struct data_output *output, char const *const *fields, int num_fields);
-    void (R_API_CALLCONV *output_flush)(struct data_output *output);
+    void (R_API_CALLCONV *output_print)(struct data_output *output, data_t *data);
     void (R_API_CALLCONV *output_free)(struct data_output *output);
+    int log_level; ///< the maximum log level (verbosity) allowed, more verbose messages must be ignored.
 } data_output_t;
 
 /** Setup known field keys and start output, used by CSV only.
