@@ -114,7 +114,7 @@ static int acurite_01185m_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return result;
 }
 
-static char *acurite_01185m_output_fields[] = {
+static char const *const acurite_01185m_output_fields[] = {
         "model",
         "id",
         "channel",
@@ -125,7 +125,7 @@ static char *acurite_01185m_output_fields[] = {
         NULL,
 };
 
-r_device acurite_01185m = {
+r_device const acurite_01185m = {
         .name        = "Acurite Grill/Meat Thermometer 01185M",
         .modulation  = OOK_PULSE_PWM,
         .short_width = 840,  // short pulse is 840 us
