@@ -28,12 +28,16 @@ Data layout:
 
 - Y is a fixed Type Code of 0x30
 - I is a device ID
-- C is the Channel number (only the lower 3 bits)
-- T is 12bits Temperature in C, scaled by 10, offset 400
+- C is 6 bits Channel number (3 bits) and flags: "1CCC0B"
+- T is 10 bits Temperature in C, scaled by 10, offset 400
 - H is Humidity
 - X is CRC-8, poly 0x31, init 0x00
 - A is SUM-8
 
+Data decoding:
+
+    TYPE:8h ID:8h ?1b CH:3b ?1b BATT:1b TEMP:10d HUM:8d CRC:8h SUM:8h ?8h8h8h8h
+
 Example packets:
 
     {177} aa aa aa aa aa aa  2d d4  30 c3 8 20a 5e  df bc   07 56 a7 ae  00 00 00 00
@@ -111,11 +115,12 @@ Seems to be the same as Fine Offset WH5360 or Ecowitt WH5360B.
 
 Data layout:
 
-    YY 00 IIII FF RRRR XX AA 00 02 ?? 00 00
+    YY 00 IIII FV RRRR XX AA 00 02 ?? 00 00
 
 - Y is a fixed Type Code of 0x40
 - I is a device ID
 - F is perhaps flags, but only seen fixed 0x10 so far
+- V is battery voltage, ( FV & 0x1f ) * 0.1f
 - R is the rain bucket tip count, 0.1mm increments
 - X is CRC-8, poly 0x31, init 0x00
 - A is SUM-8
@@ -171,15 +176,10 @@ Some payloads:
 
 static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    data_t *data;
     int events = 0;
     uint8_t b[18]; // actually only 6/9/17.5 bytes, no indication what the last 5 might be
     int row;
     int msg_type;
-    int id, channel, battery_ok, temp_raw;
-    int humidity, rain_raw;
-    float temp_c;
-    char extra[11];
     uint8_t const wh31e_type_code = 0x30; // 48
     uint8_t const wh31b_type_code = 0x37; // 55
 
@@ -209,21 +209,22 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 continue; // DECODE_FAIL_MIC
             }
 
-            id         = b[1];
-            battery_ok = (b[2] >> 7);
-            channel    = ((b[2] & 0x70) >> 4) + 1;
-            temp_raw   = ((b[2] & 0x0f) << 8) | (b[3]);
-            temp_c     = (temp_raw - 400) * 0.1f;
-            humidity   = b[4];
-            sprintf(extra, "%02x%02x%02x%02x%02x", b[6], b[7], b[8], b[9], b[10]);
+            int id       = b[1];
+            int batt_low = ((b[2] & 0x04) >> 2);
+            int channel  = ((b[2] & 0x70) >> 4) + 1;
+            int temp_raw = ((b[2] & 0x03) << 8) | (b[3]);
+            float temp_c = (temp_raw - 400) * 0.1f;
+            int humidity = b[4];
+            char extra[11];
+            snprintf(extra, sizeof(extra), "%02x%02x%02x%02x%02x", b[6], b[7], b[8], b[9], b[10]);
 
             /* clang-format off */
-            data = data_make(
+            data_t *data = data_make(
                     "model",            "",             DATA_COND, msg_type == 0x30, DATA_STRING, "AmbientWeather-WH31E",
                     "model",            "",             DATA_COND, msg_type == 0x37, DATA_STRING, "AmbientWeather-WH31B",
                     "id" ,              "",             DATA_INT,    id,
                     "channel",          "Channel",      DATA_INT,    channel,
-                    "battery_ok",       "Battery",      DATA_INT,    battery_ok,
+                    "battery_ok",       "Battery",      DATA_INT,    !batt_low,
                     "temperature_C",    "Temperature",  DATA_FORMAT, "%.1f C", DATA_DOUBLE, temp_c,
                     "humidity",         "Humidity",     DATA_FORMAT, "%u %%", DATA_INT, humidity,
                     "data",             "Extra Data",   DATA_STRING, extra,
@@ -247,7 +248,7 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 continue; // DECODE_FAIL_MIC
             }
 
-            id          = b[1];
+            int id      = b[1];
             int unknown = b[2];
             int year    = ((b[3] & 0xF0) >> 4) * 10 + (b[3] & 0x0F) + 2000;
             int month   = ((b[4] & 0x10) >> 4) * 10 + (b[4] & 0x0F);
@@ -257,11 +258,11 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             int seconds = ((b[8] & 0x70) >> 4) * 10 + (b[8] & 0x0F);
 
             char clock_str[23];
-            sprintf(clock_str, "%04d-%02d-%02dT%02d:%02d:%02dZ",
+            snprintf(clock_str, sizeof(clock_str), "%04d-%02d-%02dT%02d:%02d:%02dZ",
                     year, month, day, hours, minutes, seconds);
 
             /* clang-format off */
-            data = data_make(
+            data_t *data = data_make(
                     "model",        "",             DATA_STRING,    "AmbientWeather-WH31E",
                     "id" ,          "Station ID",   DATA_INT,       id,
                     "data",         "Unknown",      DATA_INT,       unknown,
@@ -286,23 +287,28 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 continue; // DECODE_FAIL_MIC
             }
 
-            id         = (b[2] << 8) | b[3];
-            battery_ok = (b[4] >> 7);
-            channel    = ((b[4] & 0x70) >> 4) + 1;
-            rain_raw   = (b[5] << 8) | b[6];
-            sprintf(extra, "%02x%02x%02x%02x%02x", b[9], b[10], b[11], b[12], b[13]);
+            int id         = (b[2] << 8) | b[3];
+            int battery_v  = (b[4] & 0x1f);
+            int battery_lvl = battery_v <= 9 ? 0 : ((battery_v - 9) / 6 * 100); // 0.9V-1.5V is 0-100
+            int rain_raw   = (b[5] << 8) | b[6];
+            char extra[11];
+            snprintf(extra, sizeof(extra), "%02x%02x%02x%02x%02x", b[9], b[10], b[11], b[12], b[13]);
+
+            if (battery_lvl > 100)
+                battery_lvl = 100;
 
             /* clang-format off */
-            data = data_make(
-                    "model",            "",             DATA_STRING, "EcoWitt-WH40",
-                    "id" ,              "",             DATA_INT,    id,
-                    //"channel",          "Channel",      DATA_INT,    channel,
-                    //"battery_ok",       "Battery",      DATA_INT,    battery_ok,
-                    "rain_mm",          "Total Rain",   DATA_FORMAT, "%.1f mm", DATA_DOUBLE, rain_raw * 0.1,
-                    "data",             "Extra Data",   DATA_STRING, extra,
-                    "mic",              "Integrity",    DATA_STRING, "CRC",
+            data_t *data = data_make(
+                    "model",            "",                DATA_STRING, "EcoWitt-WH40",
+                    "id" ,              "",                DATA_INT,    id,
+                    "battery_V",        "Battery Voltage", DATA_COND, battery_v != 0, DATA_FORMAT, "%f V", DATA_DOUBLE, battery_v * 0.1f,
+                    "battery_ok",       "Battery",         DATA_COND, battery_v != 0, DATA_DOUBLE, battery_lvl * 0.01f,
+                    "rain_mm",          "Total Rain",      DATA_FORMAT, "%.1f mm", DATA_DOUBLE, rain_raw * 0.1,
+                    "data",             "Extra Data",      DATA_STRING, extra,
+                    "mic",              "Integrity",       DATA_STRING, "CRC",
                     NULL);
             /* clang-format on */
+
             decoder_output_data(decoder, data);
             events++;
         }
@@ -320,21 +326,22 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 continue; // DECODE_FAIL_MIC
             }
 
-            id         = (b[2] << 8) | b[3];
-            int lux    = (b[4] << 8) | b[5];
-            int batt   = b[6];
-            battery_ok = batt > 0x30; // wild guess
-            int wspeed = b[10];
-            int wgust  = b[12];
-            int wdir   = ((b[7] & 0x20) >> 5) | b[11];
-            sprintf(extra, "%02x %02x%01x", b[13], b[16], b[17] >> 4);
+            int id      = (b[2] << 8) | b[3];
+            int lux     = (b[4] << 8) | b[5];
+            int batt    = b[6];
+            int batt_ok = batt > 0x30; // wild guess
+            int wspeed  = b[10];
+            int wgust   = b[12];
+            int wdir    = ((b[7] & 0x20) >> 5) | b[11];
+            char extra[7];
+            snprintf(extra, sizeof(extra), "%02x %02x%01x", b[13], b[16], b[17] >> 4);
 
             /* clang-format off */
-            data = data_make(
+            data_t *data = data_make(
                     "model",            "",             DATA_STRING, "EcoWitt-WS68",
                     "id" ,              "",             DATA_INT,    id,
                     "battery_raw",      "Battery Raw",  DATA_INT,    batt,
-                    "battery_ok",       "Battery",      DATA_INT,    battery_ok,
+                    "battery_ok",       "Battery",      DATA_INT,    batt_ok,
                     "lux_raw",          "lux",          DATA_INT,    lux,
                     "wind_avg_raw",     "Wind Speed",   DATA_INT,    wspeed,
                     "wind_max_raw",     "Wind Gust",    DATA_INT,    wgust,
@@ -354,11 +361,12 @@ static int ambientweather_whx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return events;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "channel",
         "battery_ok",
+        "battery_v",
         "temperature_C",
         "humidity",
         "rain_mm",
@@ -372,7 +380,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device ambientweather_wh31e = {
+r_device const ambientweather_wh31e = {
         .name        = "Ambient Weather WH31E Thermo-Hygrometer Sensor, EcoWitt WH40 rain gauge",
         .modulation  = FSK_PULSE_PCM,
         .short_width = 56,
