@@ -15,7 +15,7 @@
 #include "fatal.h"
 #include <stdlib.h>
 
-r_device fineoffset_WH2;
+r_device const fineoffset_WH2;
 
 static r_device *fineoffset_WH2_create(char *arg)
 {
@@ -157,6 +157,8 @@ static int fineoffset_WH2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 /**
 Fine Offset Electronics WH24, WH65B, HP1000 and derivatives Temperature/Humidity/Pressure sensor protocol.
 
+Also: Misol WS2320 (rebranded WH65B, 433MHz)
+
 The sensor sends a package each ~16 s with a width of ~11 ms. The bits are PCM modulated with Frequency Shift Keying.
 
 Example:
@@ -446,8 +448,9 @@ Example: 22.6 C, 40 %, 1001.7 hPa
 Data layout:
 
     aa 2d d4 e5 02 72 28 27 21 c9 bb aa
-             ?I IT TT HH PP PP CC BB
+             MI IT TT HH PP PP CC XX
 
+- M: 4 bit Model code, 0xd: old model, 0xe: new model.
 - I: 8 bit Sensor ID (based on 2 different sensors). Does not change at battery change.
 - B: 1 bit low battery indicator
 - F: 1 bit invalid reading indicator
@@ -455,7 +458,7 @@ Data layout:
 - H: 8 bit Humidity
 - P: 16 bit Pressure (*10)
 - C: 8 bit Checksum of previous 6 bytes (binary sum truncated to 8 bit)
-- B: 8 bit Bitsum (XOR) of the 6 data bytes (high and low nibble exchanged)
+- X: 8 bit Bitsum (XOR) of the 6 data bytes (high and low nibble exchanged)
 
 WH32B is the same as WH25 but two packets in one transmission of {971} and XOR sum missing.
 
@@ -627,7 +630,7 @@ static int fineoffset_WH51_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     // Decode data
     char id[7];
-    sprintf(id, "%02x%02x%02x", b[1], b[2], b[3]);
+    snprintf(id, sizeof(id), "%02x%02x%02x", b[1], b[2], b[3]);
     int boost           = (b[4] & 0xe0) >> 5;
     int battery_mv      = (b[4] & 0x1f) * 100;
     float battery_level = (battery_mv - 700) / 900.0f; // assume 1.6V (100%) to 0.7V (0%) range
@@ -785,7 +788,7 @@ static int alecto_ws1200v2_dcf_callback(r_device *decoder, bitbuffer_t *bitbuffe
     int time_m      = b[8]; // (b[8] >> 4) * 10 + (b[8] & 0x0f);
     int time_s      = b[9]; // (b[9] >> 4) * 10 + (b[9] & 0x0f);
     char clock_str[32];
-    sprintf(clock_str, "%04x-%02x-%02xT%02x:%02x:%02x",
+    snprintf(clock_str, sizeof(clock_str), "%04x-%02x-%02xT%02x:%02x:%02x",
             date_y, date_m, date_d, time_h, time_m, time_s);
 
     /* clang-format off */
@@ -950,7 +953,7 @@ static int fineoffset_WH0530_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "temperature_C",
@@ -959,7 +962,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-static char *output_fields_WH25[] = {
+static char const *const output_fields_WH25[] = {
         "model",
         "id",
         "battery_ok",
@@ -981,7 +984,7 @@ static char *output_fields_WH25[] = {
         NULL,
 };
 
-static char *output_fields_WH51[] = {
+static char const *const output_fields_WH51[] = {
         "model",
         "id",
         "battery_ok",
@@ -993,7 +996,7 @@ static char *output_fields_WH51[] = {
         NULL,
 };
 
-static char *output_fields_WH0530[] = {
+static char const *const output_fields_WH0530[] = {
         "model",
         "id",
         "battery_ok",
@@ -1004,7 +1007,7 @@ static char *output_fields_WH0530[] = {
         NULL,
 };
 
-r_device fineoffset_WH2 = {
+r_device const fineoffset_WH2 = {
         .name        = "Fine Offset Electronics, WH2, WH5, Telldus Temperature/Humidity/Rain Sensor",
         .modulation  = OOK_PULSE_PWM,
         .short_width = 500,  // Short pulse 544µs, long pulse 1524µs, fixed gap 1036µs
@@ -1016,27 +1019,27 @@ r_device fineoffset_WH2 = {
         .fields      = output_fields,
 };
 
-r_device fineoffset_WH25 = {
-        .name        = "Fine Offset Electronics, WH25, WH32B, WH24, WH65B, HP1000 Temperature/Humidity/Pressure Sensor",
+r_device const fineoffset_WH25 = {
+        .name        = "Fine Offset Electronics, WH25, WH32B, WH24, WH65B, HP1000, Misol WS2320 Temperature/Humidity/Pressure Sensor",
         .modulation  = FSK_PULSE_PCM,
-        .short_width = 58,    // Bit width = 58µs (measured across 580 samples / 40 bits / 250 kHz )
+        .short_width = 58,    // Bit width = 58µs (measured across 580 samples / 40 bits / 250 kHz)
         .long_width  = 58,    // NRZ encoding (bit width = pulse width)
         .reset_limit = 20000, // Package starts with a huge gap of ~18900 us
         .decode_fn   = &fineoffset_WH25_callback,
         .fields      = output_fields_WH25,
 };
 
-r_device fineoffset_WH51 = {
+r_device const fineoffset_WH51 = {
         .name        = "Fine Offset Electronics/ECOWITT WH51, SwitchDoc Labs SM23 Soil Moisture Sensor",
         .modulation  = FSK_PULSE_PCM,
-        .short_width = 58, // Bit width = 58µs (measured across 580 samples / 40 bits / 250 kHz )
+        .short_width = 58, // Bit width = 58µs (measured across 580 samples / 40 bits / 250 kHz)
         .long_width  = 58, // NRZ encoding (bit width = pulse width)
         .reset_limit = 5000,
         .decode_fn   = &fineoffset_WH51_callback,
         .fields      = output_fields_WH51,
 };
 
-r_device fineoffset_WH0530 = {
+r_device const fineoffset_WH0530 = {
         .name        = "Fine Offset Electronics, WH0530 Temperature/Rain Sensor",
         .modulation  = OOK_PULSE_PWM,
         .short_width = 504,  // Short pulse 504µs
