@@ -97,6 +97,7 @@ struct flex_params {
     uint32_t symbol_sync;
     struct flex_get getter[GETTER_SLOTS];
     unsigned decode_uart;
+    unsigned decode_dm;
     char const *fields[7 + GETTER_SLOTS + 1]; // NOTE: needs to match output_fields
 };
 
@@ -254,9 +255,20 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         }
     }
 
+    if (params->decode_dm) {
+        for (i = 0; i < bitbuffer->num_rows; i++) {
+            // TODO: refactor to bitbuffer_decode_dm_row()
+            unsigned len = bitbuffer->bits_per_row[i];
+            bitbuffer_t tmp = {0};
+            bitbuffer_differential_manchester_decode(bitbuffer, i, 0, &tmp, len);
+            len = tmp.bits_per_row[0];
+            memcpy(bitbuffer->bb[i], tmp.bb[0], (len + 7) / 8); // safe to write over: can only be shorter
+            bitbuffer->bits_per_row[i] = len;
+        }
+    }
+
     if (decoder->verbose) {
-        fprintf(stderr, "%s: ", params->name);
-        bitbuffer_print(bitbuffer);
+        decoder_log_bitbuffer(decoder, 1, params->name, bitbuffer, "");
     }
 
     // discard duplicates
@@ -305,6 +317,11 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         // add a data line for each getter
         render_getters(row_data[i], bitbuffer->bb[i], params);
 
+        // print at least one '0'
+        if (row_bytes[0] == '\0') {
+            snprintf(row_bytes, sizeof(row_bytes), "0");
+        }
+
         // a simpler representation for csv output
         row_codes[i] = malloc(8 + bitbuffer->bits_per_row[i] / 4 + 1); // "{nnnn}..\0"
         if (!row_codes[i])
@@ -330,7 +347,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "count",
         "num_rows",
@@ -399,6 +416,7 @@ static void help(void)
             "\tinvert : invert all bits\n"
             "\treflect : reflect each byte (MSB first to MSB last)\n"
             "\tdecode_uart : UART 8n1 (10-to-8) decode\n"
+            "\tdecode_dm : Differential Manchester decode\n"
             "\tmatch=<bits> : only match if the <bits> are found\n"
             "\tpreamble=<bits> : match and align at the <bits> preamble\n"
             "\t\t<bits> is a row spec of {<bit count>}<bits as hex number>\n"
@@ -601,10 +619,11 @@ r_device *flex_create_device(char *spec)
             if (!params->name)
                 FATAL_STRDUP("flex_create_device()");
             int name_size = strlen(val) + 27;
-            dev->name = malloc(name_size);
-            if (!dev->name)
+            char* flex_name = malloc(name_size);
+            if (!flex_name)
                 FATAL_MALLOC("flex_create_device()");
-            snprintf(dev->name, name_size, "General purpose decoder '%s'", val);
+            snprintf(flex_name, name_size, "General purpose decoder '%s'", val);
+            dev->name = flex_name;
         }
 
         else if (!strcasecmp(key, "m") || !strcasecmp(key, "modulation"))
@@ -664,6 +683,8 @@ r_device *flex_create_device(char *spec)
 
         else if (!strcasecmp(key, "decode_uart"))
             params->decode_uart = val ? atoi(val) : 1;
+        else if (!strcasecmp(key, "decode_dm"))
+            params->decode_dm = val ? atoi(val) : 1;
 
         else if (!strcasecmp(key, "symbol_zero"))
             params->symbol_zero = parse_symbol(val);
@@ -703,7 +724,7 @@ r_device *flex_create_device(char *spec)
         for (int g = 0; g < GETTER_SLOTS && params->getter[g].name; ++g) {
             params->fields[i++] = params->getter[g].name;
         }
-        dev->fields = (char **)params->fields;
+        dev->fields = params->fields;
     }
 
     // sanity checks
