@@ -146,7 +146,7 @@ static int gt_wt_03_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "channel",
@@ -158,7 +158,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device gt_wt_03 = {
+r_device const gt_wt_03 = {
         .name        = "Globaltronics GT-WT-03 Sensor",
         .modulation  = OOK_PULSE_PWM,
         .short_width = 256,
