@@ -22,8 +22,8 @@ Message is 44 bits, 11 x 4 bit nybbles:
     [00] [cnt = 10] [type] [addr] [addr + parity] [v1] [v2] [v3] [iv1] [iv2] [check]
 
 Notes:
-- Zero Pulses are longer (1,400 uS High, 1,000 uS Low) = 2,400 uS
-- One Pulses are shorter (  550 uS High, 1,000 uS Low) = 1,600 uS
+- Zero Pulses are longer (1400 uS High, 1000 uS Low) = 2400 uS
+- One Pulses are shorter (550 uS High, 1000 uS Low) = 1600 uS
 - Sensor id changes when the battery is changed
 - Primary Value are BCD with one decimal place: vvv = 12.3
 - Secondary value is integer only intval = 12, seems to be a repeat of primary
@@ -92,9 +92,8 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 parity += bit;
             }
 
-            //decoder_logf(decoder, 0, __func__, "recv: [%d/%d] %d -> msg [%d/%d] %02x, Parity: %d %s", rbyte_no, rbit_no,
-            //        bit, mnybble_no, mbit_no, msg_nybbles[mnybble_no], parity,
-            //        ( mbit_no == 0 ) ? "\n" : "" );
+            //decoder_logf(decoder, 0, __func__, "recv: [%d/%d] %d -> msg [%d/%d] %02x, Parity: %d",
+            //        rbyte_no, rbit_no, bit, mnybble_no, mbit_no, msg_nybbles[mnybble_no], parity);
         }
 
         uint8_t parity_bit = msg_nybbles[4] & 0x01;
@@ -118,10 +117,11 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
         // TODO: check if message length is a valid value
         //uint8_t msg_len      = msg_nybbles[1];
-        uint8_t msg_type     = msg_nybbles[2];
-        uint8_t sensor_id    = (msg_nybbles[3] << 3) + (msg_nybbles[4] >> 1);
-        float msg_value      = msg_nybbles[5] * 10 + msg_nybbles[6] + msg_nybbles[7] * 0.1f;
-        int msg_value_int    = msg_nybbles[8] * 10 + msg_nybbles[9];
+        uint8_t msg_type       = msg_nybbles[2];
+        uint8_t sensor_id      = (msg_nybbles[3] << 3) + (msg_nybbles[4] >> 1);
+        uint16_t msg_value_raw = (msg_nybbles[5] << 8) | (msg_nybbles[6] << 4) | msg_nybbles[7];
+        float msg_value        = msg_nybbles[5] * 10 + msg_nybbles[6] + msg_nybbles[7] * 0.1f;
+        int msg_value_int      = msg_nybbles[8] * 10 + msg_nybbles[9];
 
         // Check Repeated data values as another way of verifying
         // message integrity.
@@ -152,7 +152,7 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             data_t *data = data_make(
                     "model",            "",             DATA_STRING, "LaCrosse-TX",
                     "id",               "",             DATA_INT,    sensor_id,
-                    "humidity",         "Humidity",     DATA_FORMAT, "%.1f %%", DATA_DOUBLE, msg_value,
+                    "humidity",         "Humidity",     DATA_COND,   msg_value_raw != 0xff, DATA_FORMAT, "%.1f %%", DATA_DOUBLE, msg_value,
                     "mic",              "Integrity",    DATA_STRING, "PARITY",
                     NULL);
             /* clang-format on */
@@ -174,7 +174,7 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return result;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "temperature_C",
@@ -183,7 +183,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device lacrossetx = {
+r_device const lacrossetx = {
         .name        = "LaCrosse TX Temperature / Humidity Sensor",
         .modulation  = OOK_PULSE_PWM,
         .short_width = 550,  // 550 us pulse + 1000 us gap is 1
