@@ -98,14 +98,14 @@ static int marlec_solar_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "raw",
         "mic",
         NULL,
 };
 
-r_device marlec_solar = {
+r_device const marlec_solar = {
         .name        = "Marlec Solar iBoost+ sensors",
         .modulation  = FSK_PULSE_PCM,
         .short_width = 20,
