@@ -73,7 +73,7 @@ static int newkaku_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "unit",
@@ -84,7 +84,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device newkaku = {
+r_device const newkaku = {
         .name        = "KlikAanKlikUit Wireless Switch",
         .modulation  = OOK_PULSE_PPM,
         .short_width = 300,  // 1:1
