@@ -78,7 +78,7 @@ static int secplus_v1_decode_v1_half(r_device *decoder, uint8_t *bits, uint8_t *
                     // fprintf(stderr, "\nbin 2 = {%ld} %s\n", strlen(binstr), binstr);
                 }
                 else { // x > 3
-                    decoder_logf(decoder, 1, __func__, "Error x == %d\n", x);
+                    decoder_logf(decoder, 1, __func__, "Error x == %d", x);
                     return -1; // DECODE_FAIL_SANITY
                 }
                 x = 0;
@@ -206,7 +206,7 @@ static int secplus_v1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         gettimeofday(&cur_tv, NULL);
         timeval_subtract(&res_tv, &cur_tv, &cached_tv);
 
-        decoder_logf(decoder, 2, __func__, "res %12ld %8ld", res_tv.tv_sec, (long)res_tv.tv_usec);
+        decoder_logf(decoder, 2, __func__, "res %12ld %8ld", (long)res_tv.tv_sec, (long)res_tv.tv_usec);
 
         // is the data not expired
         if (res_tv.tv_sec == 0 && res_tv.tv_usec < CACHE_MAX_AGE) {
@@ -256,7 +256,7 @@ static int secplus_v1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     uint32_t rolling;          // max 2**32
     uint32_t rolling_temp = 0; // max 2**32
-    uint32_t fixed        = 0; // max 3^20 ( ~32 bits )
+    uint32_t fixed        = 0; // max 3^20 (~32 bits)
 
     uint8_t *res;
     res = result_1;
@@ -306,7 +306,7 @@ static int secplus_v1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     char pin_s[24] = {0};
 
     int remote_id = 0;
-    char *button  = "";
+    char const *button  = "";
 
     if (id1 == 0) {
         //  pad_id = (fixed // 3**3) % (3**7)     27  3^72187
@@ -377,7 +377,7 @@ static int secplus_v1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "id0",
@@ -395,7 +395,7 @@ static char *output_fields[] = {
 //      Freq 310.01M
 //   -X "n=v1,m=OOK_PCM,s=500,l=500,t=40,r=10000,g=7400"
 
-r_device secplus_v1 = {
+r_device const secplus_v1 = {
         .name        = "Security+ (Keyfob)",
         .modulation  = OOK_PULSE_PCM,
         .short_width = 500,
