@@ -1,5 +1,5 @@
 /** @file
-    Thermopro TP-11 Thermometer.
+    ThermoPro TP-11 Thermometer.
 
     Copyright (C) 2017 Google Inc.
 
@@ -8,18 +8,20 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 */
+
+#include "decoder.h"
+
 /**
-Thermopro TP-11 Thermometer.
+ThermoPro TP-11 Thermometer.
 
 normal sequence of bit rows:
 
-    [00] {33} db 41 57 c2 80 : 11011011 01000001 01010111 11000010 1
-    [01] {33} db 41 57 c2 80 : 11011011 01000001 01010111 11000010 1
-    [02] {33} db 41 57 c2 80 : 11011011 01000001 01010111 11000010 1
-    [03] {32} db 41 57 c2 : 11011011 01000001 01010111 11000010
+    [00] {33} db 41 57 c2 80
+    [01] {33} db 41 57 c2 80
+    [02] {33} db 41 57 c2 80
+    [03] {32} db 41 57 c2
 
 */
-#include "decoder.h"
 
 static int thermopro_tp11_sensor_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
@@ -38,8 +40,8 @@ static int thermopro_tp11_sensor_callback(r_device *decoder, bitbuffer_t *bitbuf
     }
 
     // No need to decode/extract values for simple test
-    if ( (!b[0] && !b[1] && !b[2] && !b[3])
-       || (b[0] == 0xff && b[1] == 0xff && b[2] == 0xff && b[3] == 0xff)) {
+    if ((!b[0] && !b[1] && !b[2] && !b[3])
+            || (b[0] == 0xff && b[1] == 0xff && b[2] == 0xff && b[3] == 0xff)) {
         decoder_log(decoder, 2, __func__, "DECODE_FAIL_SANITY data all 0x00 or 0xFF");
         return DECODE_FAIL_SANITY;
     }
@@ -60,7 +62,7 @@ static int thermopro_tp11_sensor_callback(r_device *decoder, bitbuffer_t *bitbuf
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "temperature_C",
@@ -68,7 +70,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device thermopro_tp11 = {
+r_device const thermopro_tp11 = {
         .name        = "Thermopro TP11 Thermometer",
         .modulation  = OOK_PULSE_PPM,
         .short_width = 500,
