@@ -1,5 +1,5 @@
 /** @file
-    Thermopro TP-12 Thermometer.
+    ThermoPro TP-12 Thermometer.
 
     Copyright (C) 2017 Google Inc.
 
@@ -8,29 +8,32 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 */
+
+#include "decoder.h"
+
 /**
-Thermopro TP-12 Thermometer.
+ThermoPro TP-12 Thermometer.
 
 A normal sequence for the TP12:
 
     [00] {0} :
-    [01] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [02] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [03] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [04] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [05] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [06] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [07] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [08] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [09] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [10] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [11] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [12] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [13] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [14] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [15] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [16] {41} 38 73 21 bb 81 80 : 00111000 01110011 00100001 10111011 10000001 1
-    [17] {40} 38 73 21 bb 81 : 00111000 01110011 00100001 10111011 10000001
+    [01] {41} 38 73 21 bb 81 80
+    [02] {41} 38 73 21 bb 81 80
+    [03] {41} 38 73 21 bb 81 80
+    [04] {41} 38 73 21 bb 81 80
+    [05] {41} 38 73 21 bb 81 80
+    [06] {41} 38 73 21 bb 81 80
+    [07] {41} 38 73 21 bb 81 80
+    [08] {41} 38 73 21 bb 81 80
+    [09] {41} 38 73 21 bb 81 80
+    [10] {41} 38 73 21 bb 81 80
+    [11] {41} 38 73 21 bb 81 80
+    [12] {41} 38 73 21 bb 81 80
+    [13] {41} 38 73 21 bb 81 80
+    [14] {41} 38 73 21 bb 81 80
+    [15] {41} 38 73 21 bb 81 80
+    [16] {41} 38 73 21 bb 81 80
+    [17] {40} 38 73 21 bb 81
 
 Layout appears to be:
 
@@ -41,11 +44,9 @@ Layout appears to be:
 
 */
 
-#include "decoder.h"
-
 #define BITS_IN_VALID_ROW 41
 
-static int thermopro_tp12_sensor_callback(r_device *decoder, bitbuffer_t *bitbuffer)
+static int thermopro_tp12_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
     int temp1_raw, temp2_raw, row;
     float temp1_c, temp2_c;
@@ -102,7 +103,7 @@ static int thermopro_tp12_sensor_callback(r_device *decoder, bitbuffer_t *bitbuf
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "id",
         "temperature_1_C",
@@ -111,13 +112,13 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device thermopro_tp12 = {
-        .name        = "Thermopro TP08/TP12/TP20 thermometer",
+r_device const thermopro_tp12 = {
+        .name        = "ThermoPro TP08/TP12/TP20 thermometer",
         .modulation  = OOK_PULSE_PPM,
         .short_width = 500,
         .long_width  = 1500,
         .gap_limit   = 2000,
         .reset_limit = 4000,
-        .decode_fn   = &thermopro_tp12_sensor_callback,
+        .decode_fn   = &thermopro_tp12_decode,
         .fields      = output_fields,
 };
