@@ -8,7 +8,10 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 */
-/** @fn int thermopro_tx2_decode(r_device *decoder, bitbuffer_t *bitbuffer)
+
+#include "decoder.h"
+
+/**
 ThermoPro TX2 sensor protocol.
 
 Note: this is the Prologue protocol with the battery low flag inverted.
@@ -35,9 +38,6 @@ The data is grouped in 9 nibbles
 - humi: 8 bit always 11001100 (0xCC) if no humidity sensor is available
 
 */
-
-#include "decoder.h"
-
 static int thermopro_tx2_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
     uint8_t *b;
@@ -93,7 +93,7 @@ static int thermopro_tx2_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     return 1;
 }
 
-static char *output_fields[] = {
+static char const *const output_fields[] = {
         "model",
         "subtype",
         "id",
@@ -105,7 +105,7 @@ static char *output_fields[] = {
         NULL,
 };
 
-r_device thermopro_tx2 = {
+r_device const thermopro_tx2 = {
         .name        = "ThermoPro-TX2 temperature sensor",
         .modulation  = OOK_PULSE_PPM,
         .short_width = 2000,
