@@ -13,6 +13,7 @@
 #include "output_mqtt.h"
 #include "optparse.h"
 #include "util.h"
+#include "logger.h"
 #include "fatal.h"
 #include "r_util.h"
 
@@ -50,7 +51,7 @@ static void mqtt_client_event(struct mg_connection *nc, int ev, void *ev_data)
         int connect_status = *(int *)ev_data;
         if (connect_status == 0) {
             // Success
-            fprintf(stderr, "MQTT Connected...\n");
+            print_log(LOG_NOTICE, "MQTT", "MQTT Connected...");
             mg_set_protocol_mqtt(nc);
             if (ctx)
                 mg_send_mqtt_handshake_opt(nc, ctx->client_id, ctx->mqtt_opts);
@@ -58,7 +59,7 @@ static void mqtt_client_event(struct mg_connection *nc, int ev, void *ev_data)
         else {
             // Error, print only once
             if (ctx && ctx->prev_status != connect_status)
-                fprintf(stderr, "MQTT connect error: %s\n", strerror(connect_status));
+                print_logf(LOG_WARNING, "MQTT", "MQTT connect error: %s", strerror(connect_status));
         }
         if (ctx)
             ctx->prev_status = connect_status;
@@ -66,35 +67,35 @@ static void mqtt_client_event(struct mg_connection *nc, int ev, void *ev_data)
     }
     case MG_EV_MQTT_CONNACK:
         if (msg->connack_ret_code != MG_EV_MQTT_CONNACK_ACCEPTED) {
-            fprintf(stderr, "MQTT Connection error: %u\n", msg->connack_ret_code);
+            print_logf(LOG_WARNING, "MQTT", "MQTT Connection error: %u", msg->connack_ret_code);
         }
         else {
-            fprintf(stderr, "MQTT Connection established.\n");
+            print_log(LOG_NOTICE, "MQTT", "MQTT Connection established.");
         }
         break;
     case MG_EV_MQTT_PUBACK:
-        fprintf(stderr, "MQTT Message publishing acknowledged (msg_id: %u)\n", msg->message_id);
+        print_logf(LOG_NOTICE, "MQTT", "MQTT Message publishing acknowledged (msg_id: %u)", msg->message_id);
         break;
     case MG_EV_MQTT_SUBACK:
-        fprintf(stderr, "MQTT Subscription acknowledged.\n");
+        print_log(LOG_NOTICE, "MQTT", "MQTT Subscription acknowledged.");
         break;
     case MG_EV_MQTT_PUBLISH: {
-        fprintf(stderr, "MQTT Incoming message %.*s: %.*s\n", (int)msg->topic.len,
+        print_logf(LOG_NOTICE, "MQTT", "MQTT Incoming message %.*s: %.*s", (int)msg->topic.len,
                 msg->topic.p, (int)msg->payload.len, msg->payload.p);
         break;
     }
     case MG_EV_CLOSE:
         if (!ctx)
-            break; // shuttig down
+            break; // shutting down
         if (ctx->prev_status == 0)
-            fprintf(stderr, "MQTT Connection failed...\n");
+            print_log(LOG_WARNING, "MQTT", "MQTT Connection lost, reconnecting...");
         // reconnect
         char const *error_string = NULL;
         ctx->connect_opts.error_string = &error_string;
         ctx->conn = mg_connect_opt(nc->mgr, ctx->address, mqtt_client_event, ctx->connect_opts);
         ctx->connect_opts.error_string = NULL;
         if (!ctx->conn) {
-            fprintf(stderr, "MQTT connect (%s) failed%s%s\n", ctx->address,
+            print_logf(LOG_WARNING, "MQTT", "MQTT connect (%s) failed%s%s", ctx->address,
                     error_string ? ": " : "", error_string ? error_string : "");
         }
         break;
@@ -114,8 +115,7 @@ static mqtt_client_t *mqtt_client_init(struct mg_mgr *mgr, tls_opts_t *tls_opts,
     //ctx->opts.keepalive = 60;
     //ctx->timeout = 10000L;
     //ctx->cleansession = 1;
-    strncpy(ctx->client_id, client_id, sizeof(ctx->client_id));
-    ctx->client_id[sizeof(ctx->client_id) - 1] = '\0';
+    snprintf(ctx->client_id, sizeof(ctx->client_id), "%s", client_id);
 
     // if the host is an IPv6 address it needs quoting
     if (strchr(host, ':'))
@@ -125,6 +125,21 @@ static mqtt_client_t *mqtt_client_init(struct mg_mgr *mgr, tls_opts_t *tls_opts,
 
     ctx->connect_opts.user_data = ctx;
     if (tls_opts && tls_opts->tls_ca_cert) {
+        print_logf(LOG_INFO, "MQTT", "mqtts (TLS) parameters are: "
+                                       "tls_cert=%s "
+                                       "tls_key=%s "
+                                       "tls_ca_cert=%s "
+                                       "tls_cipher_suites=%s "
+                                       "tls_server_name=%s "
+                                       "tls_psk_identity=%s "
+                                       "tls_psk_key=%s ",
+                tls_opts->tls_cert,
+                tls_opts->tls_key,
+                tls_opts->tls_ca_cert,
+                tls_opts->tls_cipher_suites,
+                tls_opts->tls_server_name,
+                tls_opts->tls_psk_identity,
+                tls_opts->tls_psk_key);
 #if MG_ENABLE_SSL
         ctx->connect_opts.ssl_cert          = tls_opts->tls_cert;
         ctx->connect_opts.ssl_key           = tls_opts->tls_key;
@@ -134,7 +149,7 @@ static mqtt_client_t *mqtt_client_init(struct mg_mgr *mgr, tls_opts_t *tls_opts,
         ctx->connect_opts.ssl_psk_identity  = tls_opts->tls_psk_identity;
         ctx->connect_opts.ssl_psk_key       = tls_opts->tls_psk_key;
 #else
-        fprintf(stderr, "mqtts (TLS) not available\n");
+        print_log(LOG_FATAL, __func__, "mqtts (TLS) not available");
         exit(1);
 #endif
     }
@@ -143,7 +158,7 @@ static mqtt_client_t *mqtt_client_init(struct mg_mgr *mgr, tls_opts_t *tls_opts,
     ctx->conn = mg_connect_opt(mgr, ctx->address, mqtt_client_event, ctx->connect_opts);
     ctx->connect_opts.error_string = NULL;
     if (!ctx->conn) {
-        fprintf(stderr, "MQTT connect (%s) failed%s%s\n", ctx->address,
+        print_logf(LOG_FATAL, "MQTT", "MQTT connect (%s) failed%s%s", ctx->address,
                 error_string ? ": " : "", error_string ? error_string : "");
         exit(1);
     }
@@ -219,7 +234,7 @@ static char *append_topic(char *topic, data_t *data)
         topic += sprintf(topic, "%d", data->value.v_int);
     }
     else {
-        fprintf(stderr, "Can't append data type %d to topic\n", data->type);
+        print_logf(LOG_ERROR, __func__, "Can't append data type %d to topic", data->type);
     }
 
     return topic;
@@ -282,7 +297,7 @@ static char *expand_topic(char *topic, char const *format, data_t *data, char co
         }
         // check for proper closing
         if (*format != ']') {
-            fprintf(stderr, "%s: unterminated token\n", __func__);
+            print_log(LOG_FATAL, __func__, "unterminated token");
             exit(1);
         }
         ++format;
@@ -303,7 +318,7 @@ static char *expand_topic(char *topic, char const *format, data_t *data, char co
         else if (!strncmp(t_start, "protocol", t_end - t_start))
             data_token = data_protocol;
         else {
-            fprintf(stderr, "%s: unknown token \"%.*s\"\n", __func__, (int)(t_end - t_start), t_start);
+            print_logf(LOG_FATAL, __func__, "unknown token \"%.*s\"", (int)(t_end - t_start), t_start);
             exit(1);
         }
 
@@ -484,8 +499,10 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
     // generate a short deterministic client_id to identify this input device on restart
     uint16_t host_crc = crc16((uint8_t *)mqtt->hostname, strlen(mqtt->hostname), 0x1021, 0xffff);
     uint16_t devq_crc = crc16((uint8_t *)dev_hint, dev_hint ? strlen(dev_hint) : 0, 0x1021, 0xffff);
-    char client_id[17];
-    snprintf(client_id, sizeof(client_id), "rtl_433-%04x%04x", host_crc, devq_crc);
+    uint16_t parm_crc = crc16((uint8_t *)param, param ? strlen(param) : 0, 0x1021, 0xffff);
+    char client_id[21];
+    /// MQTT 3.1.1 specifies that the broker MUST accept clients id's between 1 and 23 characters
+    snprintf(client_id, sizeof(client_id), "rtl_433-%04x%04x%04x", host_crc, devq_crc, parm_crc);
 
     // default base topic
     char base_topic[8 + sizeof(mqtt->hostname)];
@@ -503,14 +520,14 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
 
     // parse host and port
     tls_opts_t tls_opts = {0};
-    if (strncmp(param, "mqtts", 5) == 0) {
+    if (param && strncmp(param, "mqtts", 5) == 0) {
         tls_opts.tls_ca_cert = "*"; // TLS is enabled but no cert verification is performed.
     }
     param      = arg_param(param); // strip scheme
-    char *host = "localhost";
-    char *port = tls_opts.tls_ca_cert ? "8883" : "1883";
+    char const *host = "localhost";
+    char const *port = tls_opts.tls_ca_cert ? "8883" : "1883";
     char *opts = hostport_param(param, &host, &port);
-    fprintf(stderr, "Publishing MQTT data to %s port %s%s\n", host, port, tls_opts.tls_ca_cert ? " (TLS)" : "");
+    print_logf(LOG_CRITICAL, "MQTT", "Publishing MQTT data to %s port %s%s", host, port, tls_opts.tls_ca_cert ? " (TLS)" : "");
 
     // parse auth and format options
     char *key, *val;
@@ -532,11 +549,11 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
             mqtt->devices = mqtt_topic_default(val, base_topic, path_devices);
         // deprecated, remove this
         else if (!strcasecmp(key, "c") || !strcasecmp(key, "usechannel")) {
-            fprintf(stderr, "\"usechannel=...\" has been removed. Use a topic format string:\n");
-            fprintf(stderr, "for \"afterid\"   use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/id][/channel]\"\n");
-            fprintf(stderr, "for \"beforeid\"  use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/channel][/id]\"\n");
-            fprintf(stderr, "for \"replaceid\" use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/channel]\"\n");
-            fprintf(stderr, "for \"no\"        use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/id]\"\n");
+            print_log(LOG_FATAL, "MQTT", "\"usechannel=...\" has been removed. Use a topic format string:");
+            print_log(LOG_FATAL, "MQTT", "for \"afterid\"   use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/id][/channel]\"");
+            print_log(LOG_FATAL, "MQTT", "for \"beforeid\"  use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/channel][/id]\"");
+            print_log(LOG_FATAL, "MQTT", "for \"replaceid\" use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/channel]\"");
+            print_log(LOG_FATAL, "MQTT", "for \"no\"        use e.g. \"devices=rtl_433/[hostname]/devices[/type][/model][/subtype][/id]\"");
             exit(1);
         }
         // JSON events to single topic
@@ -555,7 +572,7 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
             // ok
         }
         else {
-            fprintf(stderr, "Invalid key \"%s\" option.\n", key);
+            print_logf(LOG_FATAL, __func__, "Invalid key \"%s\" option.", key);
             exit(1);
         }
     }
@@ -567,11 +584,11 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
         mqtt->states  = mqtt_topic_default(NULL, base_topic, path_states);
     }
     if (mqtt->devices)
-        fprintf(stderr, "Publishing device info to MQTT topic \"%s\".\n", mqtt->devices);
+        print_logf(LOG_NOTICE, "MQTT", "Publishing device info to MQTT topic \"%s\".", mqtt->devices);
     if (mqtt->events)
-        fprintf(stderr, "Publishing events info to MQTT topic \"%s\".\n", mqtt->events);
+        print_logf(LOG_NOTICE, "MQTT", "Publishing events info to MQTT topic \"%s\".", mqtt->events);
     if (mqtt->states)
-        fprintf(stderr, "Publishing states info to MQTT topic \"%s\".\n", mqtt->states);
+        print_logf(LOG_NOTICE, "MQTT", "Publishing states info to MQTT topic \"%s\".", mqtt->states);
 
     mqtt->output.print_data   = print_mqtt_data;
     mqtt->output.print_array  = print_mqtt_array;
@@ -582,5 +599,5 @@ struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char *param, cha
 
     mqtt->mqc = mqtt_client_init(mgr, &tls_opts, host, port, user, pass, client_id, retain, qos);
 
-    return &mqtt->output;
+    return (struct data_output *)mqtt;
 }
