@@ -14,6 +14,7 @@
 #include "data.h"
 #include "abuf.h"
 #include "r_util.h"
+#include "logger.h"
 #include "fatal.h"
 
 #include <string.h>
@@ -21,6 +22,11 @@
 #include <stdlib.h>
 
 #include <limits.h>
+// _POSIX_HOST_NAME_MAX is broken in gcc-13 at least on MacOS
+#ifndef _POSIX_HOST_NAME_MAX
+//#warning The limits.h include is missing the _POSIX_HOST_NAME_MAX define.
+#define _POSIX_HOST_NAME_MAX 255
+#endif
 // gethostname() needs _XOPEN_SOURCE 500 on unistd.h
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 500
@@ -91,7 +97,7 @@ static int datagram_client_open(datagram_client_t *client, const char *host, con
     hints.ai_flags = AI_ADDRCONFIG;
     error = getaddrinfo(host, port, &hints, &res0);
     if (error) {
-        fprintf(stderr, "%s\n", gai_strerror(error));
+        print_log(LOG_ERROR, __func__, gai_strerror(error));
         return -1;
     }
     sock = INVALID_SOCKET;
@@ -149,9 +155,8 @@ typedef struct {
     char hostname[_POSIX_HOST_NAME_MAX + 1];
 } data_output_syslog_t;
 
-static void R_API_CALLCONV print_syslog_data(data_output_t *output, data_t *data, char const *format)
+static void R_API_CALLCONV data_output_syslog_print(data_output_t *output, data_t *data)
 {
-    UNUSED(format);
     data_output_syslog_t *syslog = (data_output_syslog_t *)output;
 
     // we expect a normal message around 500 bytes
@@ -193,7 +198,7 @@ static void R_API_CALLCONV data_output_syslog_free(data_output_t *output)
     free(syslog);
 }
 
-struct data_output *data_output_syslog_create(const char *host, const char *port)
+struct data_output *data_output_syslog_create(int log_level, const char *host, const char *port)
 {
     data_output_syslog_t *syslog = calloc(1, sizeof(data_output_syslog_t));
     if (!syslog) {
@@ -210,7 +215,8 @@ struct data_output *data_output_syslog_create(const char *host, const char *port
     }
 #endif
 
-    syslog->output.print_data   = print_syslog_data;
+    syslog->output.log_level    = log_level;
+    syslog->output.output_print = data_output_syslog_print;
     syslog->output.output_free  = data_output_syslog_free;
     // Severity 5 "Notice", Facility 20 "local use 4"
     syslog->pri = 20 * 8 + 5;
@@ -229,5 +235,5 @@ struct data_output *data_output_syslog_create(const char *host, const char *port
     syslog->hostname[_POSIX_HOST_NAME_MAX] = '\0';
     datagram_client_open(&syslog->client, host, port);
 
-    return &syslog->output;
+    return (struct data_output *)syslog;
 }
