@@ -12,8 +12,11 @@
 */
 
 #include "pulse_slicer.h"
+#include "pulse_data.h"
 #include "bitbuffer.h"
 #include "util.h"
+#include "logger.h"
+#include "decoder_util.h" // TODO: this should be refactored
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -39,20 +42,27 @@ static int account_event(r_device *device, bitbuffer_t *bits, char const *demod_
         ret = 0;
     }
     else {
-        fprintf(stderr, "Decoder \"%s\" gave invalid return value %d: notify maintainer\n", device->name, ret);
+        print_logf(LOG_ERROR, demod_name, "Decoder \"%s\" gave invalid return value %d: notify maintainer", device->name, ret);
         exit(1);
     }
 
+    // Find longest row
+    unsigned max_bits = 0;
+    for (int row = 0; row < bits->num_rows; ++row) {
+        if (bits->bits_per_row[row] > max_bits) {
+            max_bits = bits->bits_per_row[row];
+        }
+    }
+
     // Debug printout
-    if (!device->decode_fn || (device->verbose && ret > 0) || device->verbose > 1) {
-        fprintf(stderr, "%s(): %s\n", demod_name, device->name);
-        bitbuffer_print(bits);
+    if (!device->decode_fn || (device->verbose && ret > 0) || (device->verbose > 1 && max_bits > 16) || (device->verbose > 2)) {
+        decoder_log_bitbuffer(device, ret > 0 ? 1 : 2, demod_name, bits, device->name);
     }
 
     return ret;
 }
 
-int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_pcm(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
     int s_short = device->short_width * samples_per_us;
@@ -69,7 +79,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -111,7 +121,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
             preamble_len = count;
             if (device->verbose > 1) {
                 float to_us = 1e6 / pulses->sample_rate;
-                fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit preamble\n",
+                print_logf(LOG_INFO, __func__, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit preamble",
                         to_us / f_long, to_us * s_long,
                         to_us / f_short, to_us * s_short, count);
             }
@@ -137,7 +147,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
         f_short = (float)rz_count / rzs_width;
         if (device->verbose > 1) {
             float to_us = 1e6 / pulses->sample_rate;
-            fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit measured\n",
+            print_logf(LOG_INFO, __func__, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit measured",
                     to_us / f_long, to_us * s_long,
                     to_us / f_short, to_us * s_short, rz_count);
         }
@@ -160,7 +170,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
             preamble_len = count;
             if (device->verbose > 1) {
                 float to_us = 1e6 / pulses->sample_rate;
-                fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f, %d bit preamble\n",
+                print_logf(LOG_INFO, __func__, "Exact bit width (in us) is %.2f vs %.2f, %d bit preamble",
                         to_us / f_short, to_us * s_short, count);
             }
         }
@@ -195,7 +205,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
         f_short = f_long = (float)nrz_count / nrz_width;
         if (device->verbose > 1) {
             float to_us = 1e6 / pulses->sample_rate;
-            fprintf(stderr, "%s: Exact bit width (in us) is %.2f vs %.2f, %d bit measured\n", device->name,
+            print_logf(LOG_INFO, __func__, "%s: Exact bit width (in us) is %.2f vs %.2f, %d bit measured", device->name,
                     to_us / f_short, to_us * s_short, nrz_count);
         }
     }
@@ -223,7 +233,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
 
             // Data is corrupt
             if (device->verbose > 3) {
-                fprintf(stderr, "bitbuffer cleared at %u: pulse %d, gap %d, period %d\n",
+                print_logf(LOG_TRACE, __func__, "bitbuffer cleared at %u: pulse %d, gap %d, period %d",
                         n, pulses->pulse[n], pulses->gap[n],
                         pulses->pulse[n] + pulses->gap[n]);
             }
@@ -246,7 +256,7 @@ int pulse_slicer_pcm(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_ppm(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_ppm(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -264,7 +274,7 @@ int pulse_slicer_ppm(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -325,7 +335,7 @@ int pulse_slicer_ppm(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_pwm(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_pwm(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -343,7 +353,7 @@ int pulse_slicer_pwm(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -438,7 +448,7 @@ int pulse_slicer_pwm(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_manchester_zerobit(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_manchester_zerobit(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -456,7 +466,7 @@ int pulse_slicer_manchester_zerobit(const pulse_data_t *pulses, r_device *device
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -517,7 +527,7 @@ int pulse_slicer_manchester_zerobit(const pulse_data_t *pulses, r_device *device
     return events;
 }
 
-static inline int pulse_slicer_get_symbol(const pulse_data_t *pulses, unsigned int n)
+static inline int pulse_slicer_get_symbol(pulse_data_t const *pulses, unsigned int n)
 {
     if (n % 2 == 0)
         return pulses->pulse[n / 2];
@@ -525,7 +535,7 @@ static inline int pulse_slicer_get_symbol(const pulse_data_t *pulses, unsigned i
         return pulses->gap[n / 2];
 }
 
-int pulse_slicer_dmc(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_dmc(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -543,7 +553,7 @@ int pulse_slicer_dmc(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -556,7 +566,7 @@ int pulse_slicer_dmc(const pulse_data_t *pulses, r_device *device)
         if (abs(symbol - s_short) < s_tolerance) {
             // Short - 1
             bitbuffer_add_bit(&bits, 1);
-            symbol = pulse_slicer_get_symbol(pulses, ++n);
+            symbol = n + 1 < pulses->num_pulses * 2 ? pulse_slicer_get_symbol(pulses, ++n) : 0;
             if (abs(symbol - s_short) > s_tolerance) {
                 if (symbol >= s_reset - s_tolerance) {
                     // Don't expect another short gap at end of message
@@ -565,7 +575,7 @@ int pulse_slicer_dmc(const pulse_data_t *pulses, r_device *device)
                 else if (bits.num_rows > 0 && bits.bits_per_row[bits.num_rows - 1] > 0) {
                     bitbuffer_add_row(&bits);
 /*
-                    fprintf(stderr, "Detected error during pulse_slicer_dmc(): %s\n",
+                    print_logf(LOG_WARNING, __func__, "Detected error during pulse_slicer_dmc(): %s",
                             device->name);
 */
                 }
@@ -585,7 +595,7 @@ int pulse_slicer_dmc(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_piwm_raw(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_piwm_raw(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -603,7 +613,7 @@ int pulse_slicer_piwm_raw(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -631,7 +641,7 @@ int pulse_slicer_piwm_raw(const pulse_data_t *pulses, r_device *device)
                 && bits.bits_per_row[bits.num_rows - 1] > 0) {
             bitbuffer_add_row(&bits);
 /*
-            fprintf(stderr, "Detected error during pulse_slicer_piwm_raw(): %s\n",
+            print_logf(LOG_WARNING, __func__, "Detected error during pulse_slicer_piwm_raw(): %s",
                     device->name);
 */
         }
@@ -647,7 +657,7 @@ int pulse_slicer_piwm_raw(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_piwm_dc(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_piwm_dc(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -665,7 +675,7 @@ int pulse_slicer_piwm_dc(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -687,7 +697,7 @@ int pulse_slicer_piwm_dc(const pulse_data_t *pulses, r_device *device)
                 && bits.bits_per_row[bits.num_rows - 1] > 0) {
             bitbuffer_add_row(&bits);
 /*
-            fprintf(stderr, "Detected error during pulse_slicer_piwm_dc(): %s\n",
+            print_logf(LOG_WARNING, __func__, "Detected error during pulse_slicer_piwm_dc(): %s",
                     device->name);
 */
         }
@@ -703,7 +713,7 @@ int pulse_slicer_piwm_dc(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
-int pulse_slicer_nrzs(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_nrzs(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -721,7 +731,7 @@ int pulse_slicer_nrzs(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -763,7 +773,7 @@ int pulse_slicer_nrzs(const pulse_data_t *pulses, r_device *device)
  * bit is discarded.
  */
 
-int pulse_slicer_osv1(const pulse_data_t *pulses, r_device *device)
+int pulse_slicer_osv1(pulse_data_t const *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
 
@@ -781,7 +791,7 @@ int pulse_slicer_osv1(const pulse_data_t *pulses, r_device *device)
             || (device->gap_limit > 0 && s_gap <= 0)
             || (device->sync_width > 0 && s_sync <= 0)
             || (device->tolerance > 0 && s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", device->protocol_num, device->name);
+        print_logf(LOG_WARNING, __func__, "sample rate too low for protocol %u \"%s\"", device->protocol_num, device->name);
         return 0;
     }
 
@@ -806,7 +816,7 @@ int pulse_slicer_osv1(const pulse_data_t *pulses, r_device *device)
     }
     if (preamble != 12) {
         if (device->verbose)
-            fprintf(stderr, "preamble %d  %d %d\n", preamble, pulses->pulse[0], pulses->gap[0]);
+            print_logf(LOG_WARNING, __func__, "preamble %d  %d %d", preamble, pulses->pulse[0], pulses->gap[0]);
         return events;
     }
 
